"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CoreCronService = void 0;
const common_1 = require("@nestjs/common");
const core_cron_entity_1 = require("./entities/core-cron.entity");
const constants_1 = require("../../common/constants");
const cron_1 = require("cron");
const fs = require("fs");
const core_scripts_service_1 = require("../core-scripts/core-scripts.service");
const common_2 = require("../../common");
const msgtemplate_service_1 = require("../../business/communications/msgtemplate/msgtemplate.service");
const schedule_1 = require("@nestjs/schedule");
let CoreCronService = class CoreCronService extends common_2.BaseService {
    constructor(model, coreCron, coreScriptService, utilityService, msgTemplateService, schedulerRegistry) {
        super(model);
        this.model = model;
        this.coreCron = coreCron;
        this.coreScriptService = coreScriptService;
        this.utilityService = utilityService;
        this.msgTemplateService = msgTemplateService;
        this.schedulerRegistry = schedulerRegistry;
    }
    refreshCron() {
        return __awaiter(this, void 0, void 0, function* () {
            const jobs = yield this.schedulerRegistry.getCronJobs();
            jobs.forEach((value, key, map) => {
                let next;
                try {
                    next = value.nextDates().toDate();
                    const job = this.schedulerRegistry.getCronJob(key);
                    job.stop();
                    console.log(`job: ${key} -> stopped: ${next}`);
                }
                catch (e) {
                    console.log(e);
                    next = "error: next fire date is in the past!";
                }
            });
            return jobs;
        });
    }
    initializeCron() {
        return __awaiter(this, void 0, void 0, function* () {
            const cronData = yield this.coreCron.findAll();
            cronData.forEach((cronConfiguration) => __awaiter(this, void 0, void 0, function* () {
                this.addNewJob({
                    values: {
                        cronConfiguration: cronConfiguration,
                    },
                });
            }));
        });
    }
    groupingLogic(data, configuration) {
        return __awaiter(this, void 0, void 0, function* () {
            const groupedData = this.utilityService.groupBy(data[0], configuration.group_by);
            let headings = [];
            data[0].map((element) => __awaiter(this, void 0, void 0, function* () {
                Object.keys(element).forEach((field) => {
                    headings.push(field);
                });
            }));
            let templateContent = Object.keys(groupedData).map((index) => {
                return {
                    [configuration.group_by]: index,
                    results: groupedData[index],
                    headings: headings,
                };
            });
            return templateContent;
        });
    }
    addNewJob({ values }) {
        return __awaiter(this, void 0, void 0, function* () {
            const job = new cron_1.CronJob(`${values.cronConfiguration.cron_pattern}`, () => __awaiter(this, void 0, void 0, function* () {
                this.executeCron(values.cronConfiguration);
            }));
            this.schedulerRegistry.addCronJob(values.cronConfiguration.description, job);
            job.start();
        });
    }
    executeCron(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const cronConfiguration = yield this.findOne(id);
            const script_id = cronConfiguration.script_id;
            const dynamicScript = "script";
            const ids = null;
            const transaction = null;
            let templateContent;
            let additionalConfiguration = null;
            let messageTemplate = null;
            messageTemplate = yield this.msgTemplateService.getMessageTemplate(cronConfiguration);
            const data = yield this.coreScriptService.executeScript(script_id, dynamicScript, id, transaction);
            templateContent = data[0];
            if (messageTemplate) {
                additionalConfiguration = JSON.parse(messageTemplate.mt_otherdetails);
                if (additionalConfiguration.group_by) {
                    templateContent = yield this.groupingLogic(data, additionalConfiguration);
                }
            }
            switch (additionalConfiguration.type) {
                case "pdf":
                    yield this.msgTemplateService.generateCronwithPdfAttachment({
                        values: {
                            cronConfiguration: cronConfiguration,
                            messageTemplate: messageTemplate,
                            templateContent: templateContent,
                            otherDetails: additionalConfiguration,
                        },
                    });
                    break;
                case "excel":
                    console.log("Code of excel needs to be updated");
                    break;
                default:
                    yield this.msgTemplateService.cronMail({
                        values: {
                            cronConfiguration: cronConfiguration,
                            messageTemplate: messageTemplate,
                            templateContent: templateContent,
                            otherDetails: additionalConfiguration,
                        },
                        attachment: null,
                    });
            }
        });
    }
};
CoreCronService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)(constants_1.CORE_CRON)),
    __param(1, (0, common_1.Inject)(constants_1.CORE_CRON)),
    __metadata("design:paramtypes", [core_cron_entity_1.CoreCron, Object, core_scripts_service_1.CoreScriptsService,
        common_2.UtilityService,
        msgtemplate_service_1.MsgTemplateService,
        schedule_1.SchedulerRegistry])
], CoreCronService);
exports.CoreCronService = CoreCronService;
