"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTenancyProviders = void 0;
const tenancy_constants_1 = require("../tenancy.constants");
const utils_1 = require("../utils");
const createTenancyProviders = (definitions) => {
    const providers = [];
    for (const definition of definitions) {
        const { name, schema, injectName } = definition;
        providers.push({
            provide: (0, utils_1.getTenantModelDefinitionToken)(name),
            useFactory: (modelDefinitionMap, connectionMap) => {
                const exists = modelDefinitionMap.has(name);
                if (!exists) {
                    modelDefinitionMap.set(name, Object.assign({}, definition));
                    connectionMap.forEach((connection) => {
                        connection.addModels([schema]);
                    });
                }
            },
            inject: [tenancy_constants_1.MODEL_DEFINITION_MAP, tenancy_constants_1.CONNECTION_MAP],
        });
        providers.push({
            provide: (0, utils_1.getTenantModelToken)(injectName),
            useFactory: (tenantConnection) => __awaiter(void 0, void 0, void 0, function* () {
                yield tenantConnection.addModels([schema]);
                return tenantConnection.models[name];
            }),
            inject: [tenancy_constants_1.TENANT_CONNECTION],
        });
    }
    return providers;
};
exports.createTenancyProviders = createTenancyProviders;
